#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Acquia Applications resource"""

from acapi2.exceptions import AcquiaCloudNoDataException
from acapi2.resources.acquialist import AcquiaList
from acapi2.resources.application import Application


class ApplicationList(AcquiaList):
    def __init__(
        self,
        uri: str,
        api_key: str,
        api_secret: str,
        qry_params: dict = None,
        *args,
        **kwargs,
    ) -> None:

        super().__init__(uri, api_key, api_secret, *args, **kwargs)
        self._qry_params = qry_params
        self.fetch()

    def fetch(self) -> None:
        apps = self.request(uri=self.uri, params=self._qry_params).json()
        try:
            app_items = apps["_embedded"]["items"]
        except KeyError:
            raise AcquiaCloudNoDataException()
        else:
            index = 0
            for app in app_items:
                subs_uri = app["_links"]["self"]["href"]
                self.__setitem__(
                    index, Application(subs_uri, self.api_key, self.api_secret)
                )
                index += 1

    @property
    def base_uri(self) -> str:
        return self._base_uri

    @base_uri.setter
    def base_uri(self, base_uri: str):
        self._base_uri = f"{base_uri}/applications"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Acquia Cloud API Exceptions"""

from pprint import pformat


class AcquiaCloudException(Exception):
    """Generic Acquia Cloud API Exception.

    All acapi2 exceptions should extend this class.
    """


class AcquiaCloudNoDataException(AcquiaCloudException):
    """No data found exception."""


class AcquiaCloudNotificationException(AcquiaCloudException):
    """An Acquia notification exception."""

    def __init__(self, message, task):
        """Constructor.

        Parameters
        ----------
        message: str
            The error message.
        task: Task
            The Task object for the task that failed.
        """
        super().__init__(message)
        self.message = message
        self.task = task

    def __str__(self):
        """Return the string representation of the exception.

        Returns
        -------
        str
            The error message and pretty printed Task object properties.
        """
        task = pformat(self.task, indent=4)
        return f"{self.message}\n{task}"


class AcquiaCloudTaskFailedException(AcquiaCloudNotificationException):
    """An Acquia task failure exception."""


class AcquiaCloudNotificationFailedException(AcquiaCloudNotificationException):
    """An Acquia notification failure exception."""


class AcquiaCloudTimeoutError(AcquiaCloudNotificationException):
    """Timeout exceeded error."""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""ACAPI2 Tests."""
import unittest

import requests
import requests_mock

from acapi2 import Acquia


class BaseTest(unittest.TestCase):
    acquia = None

    def setUp(self):
        self.endpoint = "https://cloud.acquia.com/api"

        # These api key and secret examples
        # were taken from the official API docs
        api_key = "d0697bfc-7f56-4942-9205-b5686bf5b3f5"
        api_secret = "D5UfO/4FfNBWn4+0cUwpLOoFzfP7Qqib4AoY+wYGsKE="

        session = requests.session()
        adapter = requests_mock.Adapter()

        session.mount("mock", adapter)
        self.acquia = Acquia(api_key, api_secret)



import json
from typing import Dict


def load_fixture(path: str) -> Dict:
    """Load fixture from a json-file.

    :param path: path to the file
    :return: formatted dict
    """
    with open(path) as f:
        result = f.read()
    return json.loads(result)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Acquia Main Object"""
import os

from acapi2.exceptions import AcquiaCloudException
from acapi2.resources.agreement import Agreement
from acapi2.resources.agreementlist import AgreementList
from acapi2.resources.application import Application
from acapi2.resources.applicationlist import ApplicationList
from acapi2.resources.environment import Environment
from acapi2.resources.notification import Notification
from acapi2.resources.permissionslist import PermissionsList
from acapi2.resources.subscription import Subscription
from acapi2.resources.subscriptionlist import SubscriptionList


class Acquia(object):
    _api_endpoint = "https://cloud.acquia.com/api"

    def __init__(
        self, api_key: str = None, api_secret: str = None, endpoint: str = None
    ) -> None:

        if endpoint:
            self._api_endpoint = endpoint

        if not api_key or not api_secret:
            api_key, api_secret = self.__find_credentials()
            if not api_key or not api_secret:
                msg = "Credentials not provided"
                raise AcquiaCloudException(msg)

        self._api_key = api_key
        self._api_secret = api_secret

    def agreements(self) -> AgreementList:
        return AgreementList(self.api_endpoint, self.api_key, self.api_secret)

    def agreement(self, uuid) -> Agreement:
        namespace = f"agreements/{uuid}"
        uri = self.get_uri(namespace)
        return Agreement(uri, self.api_key, self.api_secret)

    def applications(
        self,
        filters: str = None,
        sort: str = None,
        limit: int = None,
        offset: int = None,
    ) -> ApplicationList:
        qry_params = {
            "filter": filters,
            "sort": sort,
            "limit": limit,
            "offset": offset,
        }

        apps = ApplicationList(
            self.api_endpoint,
            self.api_key,
            self.api_secret,
            qry_params=qry_params,
        )
        return apps

    def application(self, uuid: str) -> Application:
        namespace = f"applications/{uuid}"
        uri = self.get_uri(namespace)
        application = Application(uri, self.api_key, self.api_secret)
        return application

    def notification(self, uuid: str) -> Notification:
        namespace = f"notifications/{uuid}"
        uri = self.get_uri(namespace)
        return Notification(uri, self.api_key, self.api_secret)

    def environment(self, env_id: str) -> Environment:
        namespace = "environments/" + str(env_id)
        uri = self.get_uri(namespace)
        return Environment(uri, self.api_key, self.api_secret)

    @property
    def api_endpoint(self) -> str:
        return self._api_endpoint

    @api_endpoint.setter
    def api_endpoint(self, endpoint: str):
        self._api_endpoint = endpoint

    @property
    def api_key(self) -> str:
        return self._api_key

    @api_key.setter
    def api_key(self, api_key: str):
        # TODO: validate api_key
        self._api_key = api_key

    @property
    def api_secret(self) -> str:
        return self._api_secret

    @api_secret.setter
    def api_secret(self, api_secret: str):
        # TODO: validate api_secret
        self._api_secret = api_secret

    def health(self):
        raise NotImplementedError

    def get_uri(self, path: str) -> str:
        return f"{self.api_endpoint}/{path}"

    def permissions(self) -> PermissionsList:
        return PermissionsList(
            self.api_endpoint, self.api_key, self.api_secret
        )

    def subscription(self, uuid: str) -> Subscription:
        namespace = "subscriptions/" + uuid
        uri = self.get_uri(namespace)
        subscription = Subscription(uri, self.api_key, self.api_secret)
        return subscription

    def subscriptions(self, filters: dict = None) -> SubscriptionList:
        subs = SubscriptionList(
            self.api_endpoint, self.api_key, self.api_secret, filters=filters
        )
        return subs

    @staticmethod
    def __find_credentials() -> tuple:
        key = os.environ.get("ACQUIA_CLOUD_API_KEY")
        secret = os.environ.get("ACQUIA_CLOUD_API_SECRET")
        return key, secret



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import unittest
from unittest.mock import patch

import requests
import requests_mock

from acapi2.http_request import HttpRequest
from acapi2.resources.acquiadata import AcquiaData


@requests_mock.Mocker()
class TestAcquiaData(unittest.TestCase):
    endpoint = "https://cloud.acquia.com/api/"

    # These api key and secret examples
    # were taken from the official API docs
    api_key = "d0697bfc-7f56-4942-9205-b5686bf5b3f5"
    api_secret = "D5UfO/4FfNBWn4+0cUwpLOoFzfP7Qqib4AoY+wYGsKE="

    def test_endpoint_not_found(self, mocker):
        uri = self.endpoint + "invalid/"

        mocker.register_uri("GET", uri, status_code=404)

        acquia_data = AcquiaData(uri, self.api_key, self.api_secret)

        with self.assertRaises(requests.HTTPError):
            acquia_data.request()

    def test_retry_and_get(self, mocker):
        mocker.register_uri(
            "GET",
            self.endpoint,
            [
                {"status_code": 500},
                {"status_code": 503},
                {"status_code": 502},
                {"status_code": 200},
            ],
        )

        acquia_data = AcquiaData(self.endpoint, self.api_key, self.api_secret)

        self.assertEqual(acquia_data.request().status_code, 200)

    def test_unauthorised_request(self, mocker):
        mocker.register_uri("GET", self.endpoint, status_code=403)
        acquia_data = AcquiaData(self.endpoint, self.api_key, self.api_secret)

        with self.assertRaises(requests.HTTPError):
            acquia_data.request()

    def test_post_request(self, mocker):
        uri = self.endpoint + "organizations"
        mocker.register_uri(
            "POST",
            uri,
            status_code=200,
            json={"message": "Oganization created."},
        )

        acquia_data = AcquiaData(uri, self.api_key, self.api_secret)

        rdata = {"name": "A Random name"}
        response = acquia_data.request(method="POST", data=rdata)

        self.assertEqual(response.status_code, 200)
        self.assertIn(b"created", response.content)

    def test_delete_request(self, mocker):
        uri = self.endpoint + "organizations/1234"
        acquia_response = {
            "id": "6",
            "uuid": "g47ac10b-58cc-4372-a567-0e02b2c3d470",
            "name": "Sample organization",
            "subscriptions_total": "115",
            "admins_total": "2",
            "users_total": "82",
            "teams_total": "13",
            "roles_total": "4",
        }
        mocker.register_uri(
            "DELETE", uri, status_code=202, json=acquia_response
        )

        acquia_data = AcquiaData(uri, self.api_key, self.api_secret)
        response = acquia_data.request(method="DELETE")

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"id", response.content)
        self.assertIn(b"name", response.content)

    @patch.object(HttpRequest, "do")
    def test_request_backoff(self, mocker, request_do_mock):
        request_do_mock.side_effect = [
            requests.exceptions.ConnectionError(),
            requests_mock.create_response(requests.Request()),
        ]

        acquia_data = AcquiaData(self.endpoint, self.api_key, self.api_secret)
        acquia_data.request()

        assert request_do_mock.call_count == 2



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Test Notification Endpoint"""

from unittest.mock import patch

import requests_mock

from acapi2.exceptions import (
    AcquiaCloudNotificationFailedException,
    AcquiaCloudTimeoutError,
)
from acapi2.resources.notification import Notification
from acapi2.tests import BaseTest
from acapi2.tests.util import load_fixture


@requests_mock.Mocker()
class TestNotification(BaseTest):
    def test_pending(self, mocker):
        response = load_fixture("acapi2/tests/fixtures/notification.json")

        notification_uuid = "1bd3487e-71d1-4fca-a2d9-5f969b3d35c1"
        uri = f"{self.endpoint}/notifications/{notification_uuid}"
        mocker.register_uri("GET", uri, status_code=200, json=response)
        is_pending = self.acquia.notification(notification_uuid).pending()
        assert is_pending

    @patch.object(Notification, "pending", return_value=True)
    def test_wait_in_progress(self, mocker, mock_pending):
        response = load_fixture("acapi2/tests/fixtures/notification.json")

        notification_uuid = "1bd3487e-71d1-4fca-a2d9-5f969b3d35c1"
        uri = f"{self.endpoint}/notifications/{notification_uuid}"
        mocker.register_uri("GET", uri, status_code=200, json=response)
        with self.assertRaises(AcquiaCloudTimeoutError):
            self.acquia.notification(notification_uuid).wait(5)

    @patch.object(Notification, "pending", return_value=False)
    def test_wait_failed(self, mocker, mock_pending):
        response = load_fixture(
            "acapi2/tests/fixtures/notification_failed.json"
        )

        notification_uuid = "1bd3487e-71d1-4fca-a2d9-5f969b3d35c1"
        uri = f"{self.endpoint}/notifications/{notification_uuid}"
        mocker.register_uri("GET", uri, status_code=200, json=response)
        with self.assertRaises(AcquiaCloudNotificationFailedException):
            self.acquia.notification(notification_uuid).wait(5)

    @patch.object(Notification, "pending", return_value=False)
    def test_wait_completed(self, mocker, mock_pending):
        response = load_fixture(
            "acapi2/tests/fixtures/notification_completed.json"
        )

        notification_uuid = "1bd3487e-71d1-4fca-a2d9-5f969b3d35c1"
        uri = f"{self.endpoint}/notifications/{notification_uuid}"
        mocker.register_uri("GET", uri, status_code=200, json=response)
        notification = self.acquia.notification(notification_uuid).wait(5)
        self.assertIsInstance(notification, Notification)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Test Notifications Endpoint"""

import requests_mock

from acapi2.resources.notificationlist import NotificationList
from acapi2.tests import BaseTest


@requests_mock.Mocker()
class TestNotifications(BaseTest):
    def test_notifications(self, mocker):
        response = {
            "_embedded": {
                "items": [
                    {
                        "_embedded": {
                            "author": {
                                "created_at": "2019-11-25T14:07:44+00:00",
                                "email": "some@email.com",
                                "first_name": "Name",
                                "last_login_at": "2020-02-07T12:24:29+00:00",
                                "last_name": "Ue",
                                "picture_url": "https://accounts.acquia.com/"
                                "images/users/f0886584-0830-55"
                                "5d-c234-14ffde052431/"
                                "style/avatar",
                                "username": "username",
                                "uuid": "f0886584-0830-555d-c234-"
                                "14ffde052431",
                            }
                        },
                        "_links": {
                            "parent": {
                                "href": "https://cloud.acquia.com/api"
                                "/notifications"
                            },
                            "self": {
                                "href": "https://cloud.acquia.com/api/"
                                "notifications/1bd3487e-71d1-4fca-a2d9"
                                "-5f969b3d35c1"
                            },
                        },
                        "completed_at": "2020-02-06T13:33:08+00:00",
                        "created_at": "2020-02-06T13:31:48+00:00",
                        "description": "Log forwarding destination created on Dev.",
                        "event": "LogForwardingDestinationCreated",
                        "label": "Log forwarding destination created",
                        "progress": 100,
                        "status": "completed",
                        "uuid": "1bd3487e-71d1-4fca-a2d9-5f969b3d35c1",
                    }
                ]
            },
            "_links": {
                "filter": {
                    "href": "https://cloud.acquia.com/api/applications/"
                    "f027502b-ed6c-448e-97e8-4a0def7d25e1/"
                    "notifications{?filter}",
                    "templated": True,
                },
                "limit": {
                    "href": "https://cloud.acquia.com/api/"
                    "applications/f027502b-ed6c-448e-97e8-"
                    "4a0def7d25e1/notifications{?limit}",
                    "templated": True,
                },
                "next": {
                    "href": "https://cloud.acquia.com/api/applications/"
                    "f027502b-ed6c-448e-97e8-4a0def7d25e1/"
                    "notifications?limit=2&offset=2"
                },
                "offset": {
                    "href": "https://cloud.acquia.com/api/applications/"
                    "f027502b-ed6c-448e-97e8-4a0def7d25e1/"
                    "notifications{?offset}",
                    "templated": True,
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/applications/"
                    "f027502b-ed6c-448e-97e8-4a0def7d25e1?limit=2"
                },
                "self": {
                    "href": "https://cloud.acquia.com/api/applications/"
                    "f027502b-ed6c-448e-97e8-4a0def7d25e1/"
                    "notifications?limit=2"
                },
                "sort": {
                    "href": "https://cloud.acquia.com/api/applications/"
                    "f027502b-ed6c-448e-97e8-4a0def7d25e1/"
                    "notifications{?sort}",
                    "templated": True,
                },
            },
            "pagination": {"limit": 2, "offset": 0, "total": "81"},
            "total": "81",
        }

        app_uuid = "f027502b-ed6c-448e-97e8-4a0def7d25e1"
        uri = f"{self.endpoint}/applications/{app_uuid}/notifications"
        mocker.register_uri("GET", uri, status_code=200, json=response)
        notif = self.acquia.application(app_uuid).notifications()

        self.assertIsInstance(notif, NotificationList)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Test Acquia Environments"""

import requests_mock

from acapi2.tests import BaseTest


@requests_mock.Mocker()
class TestEnvironments(BaseTest):
    def test_backups(self, mocker):
        env_id = "1-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        db_name = "db_name"
        uri = (
            f"{self.endpoint}/environments/{env_id}/"
            f"databases/{db_name}/backups"
        )

        response = {
            "total": 2,
            "pagination": {"total": 2, "limit": 2, "offset": 0},
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "1-a47ac10b-58cc-4372-a567-0e02b2c3d470/"
                    "databases/db_name/backups"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "1-a47ac10b-58cc-4372-a567-0e02b2c3d470/"
                    "databases/db_name"
                },
                "limit": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "1-a47ac10b-58cc-4372-a567-0e02b2c3d470/databases"
                    "/db_name/backups{?limit}",
                    "templated": True,
                },
                "offset": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "1-a47ac10b-58cc-4372-a567-0e02b2c3d470/databases/"
                    "db_name/backups{?offset}",
                    "templated": True,
                },
                "sort": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "1-a47ac10b-58cc-4372-a567-0e02b2c3d470/"
                    "databases/db_name/backups{?sort}",
                    "templated": True,
                },
                "filter": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "1-a47ac10b-58cc-4372-a567-0e02b2c3d470/d"
                    "atabases/db_name/backups{?filter}",
                    "templated": True,
                },
            },
            "_embedded": {
                "items": [
                    {
                        "id": 1,
                        "database": {"id": 14, "name": "db_name"},
                        "type": "daily",
                        "started_at": "2012-05-15T12:00:00Z",
                        "completed_at": "2012-05-15T12:00:00Z",
                        "flags": {"deleted": False},
                        "environment": {
                            "id": "1-a47ac10b-58cc-4372-a567-0e02b2c3d470",
                            "name": "Production",
                        },
                        "_links": {
                            "self": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/1-a47ac10b-58cc-4372-"
                                "a567-0e02b2c3d470/databases/"
                                "db_name/backups/1"
                            },
                            "parent": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/1-a47ac10b-58cc-"
                                "4372-a567-0e02b2c3d470/databases"
                            },
                            "download": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/1-a47ac10b-58cc-4372"
                                "-a567-0e02b2c3d470/databases/"
                                "db_name/backups/1/actions/download"
                            },
                        },
                    },
                    {
                        "id": 2,
                        "database": {"id": 14, "name": "db_name"},
                        "type": "daily",
                        "started_at": "2012-03-28T12:00:00Z",
                        "completed_at": "2012-03-28T12:00:01Z",
                        "flags": {"deleted": False},
                        "environment": {
                            "id": "1-a47ac10b-58cc-4372-a567-0e02b2c3d470",
                            "name": "Production",
                        },
                        "_links": {
                            "self": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/1-a47ac10b-58cc-4372-"
                                "a567-0e02b2c3d470/databases/"
                                "db_name/backups/2"
                            },
                            "parent": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/1-a47ac10b-58cc-4372-"
                                "a567-0e02b2c3d470/databases"
                            },
                            "download": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/1-a47ac10b-58cc-4372-"
                                "a567-0e02b2c3d470/databases/db_name"
                                "/backups/2/actions/download"
                            },
                        },
                    },
                    {
                        "id": 3,
                        "database": {"id": 14, "name": "db_name"},
                        "type": "daily",
                        "started_at": "2017-01-08T04:00:00Z",
                        "completed_at": "2017-01-08T04:00:01Z",
                        "flags": {"deleted": False},
                        "environment": {
                            "id": "1-a47ac10b-58cc-4372-a567-0e02b2c3d470",
                            "name": "Production",
                        },
                        "_links": {
                            "self": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/1-a47ac10b-58cc-4372-"
                                "a567-0e02b2c3d470/databases/db_name"
                                "/backups/3"
                            },
                            "parent": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/1-a47ac10b-58cc-4372-"
                                "a567-0e02b2c3d470/databases"
                            },
                            "download": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/1-a47ac10b-58cc-4372-"
                                "a567-0e02b2c3d470/databases/db_name"
                                "/backups/3/actions/download"
                            },
                        },
                    },
                    {
                        "id": 4,
                        "database": {"id": 14, "name": "db_name"},
                        "type": "daily",
                        "started_at": "2017-01-08T05:00:02Z",
                        "completed_at": "2017-01-08T05:00:03Z",
                        "flags": {"deleted": False},
                        "environment": {
                            "id": "1-a47ac10b-58cc-4372-a567-0e02b2c3d470",
                            "name": "Production",
                        },
                        "_links": {
                            "self": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/1-a47ac10b-58cc-4372-"
                                "a567-0e02b2c3d470/databases/db_name"
                                "/backups/4"
                            },
                            "parent": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/1-a47ac10b-58cc-4372-"
                                "a567-0e02b2c3d470/databases"
                            },
                            "download": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/1-a47ac10b-58cc-4372-"
                                "a567-0e02b2c3d470/databases/db_name"
                                "/backups/4/actions/download"
                            },
                        },
                    },
                ]
            },
        }

        mocker.register_uri("GET", uri, status_code=200, json=response)

        response = self.acquia.environment(env_id).backups(db_name)
        self.assertEqual(response["total"], 2)
        self.assertIn("_embedded", response)

    def test_backup_details(self, mocker):
        env_id = "1-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        db_name = "db_name"
        id_backup = "1"
        uri = (
            f"{self.endpoint}/environments/{env_id}/"
            f"databases/{db_name}/backups/{id_backup}"
        )

        response = {
            "id": 1,
            "database": {"id": 14, "name": "db_name"},
            "type": "daily",
            "started_at": "2012-05-15T12:00:00Z",
            "completed_at": "2012-05-15T12:00:00Z",
            "flags": {"deleted": False},
            "environment": {
                "id": "1-a47ac10b-58cc-4372-a567-0e02b2c3d470",
                "name": "Production",
            },
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "1-a47ac10b-58cc-4372-a567-0e02b2c3d470/"
                    "database-backups/1"
                },
                "download": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "1-a47ac10b-58cc-4372-a567-0e02b2c3d470/"
                    "database-backups/1/actions/download"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "1-a47ac10b-58cc-4372-a567-0e02b2c3d470/d"
                    "atabase-backups"
                },
            },
            "_embedded": {
                "environment": {
                    "id": "1-a47ac10b-58cc-4372-a567-0e02b2c3d470",
                    "name": "Production",
                    "_links": {
                        "self": {
                            "href": "https://cloud.acquia.com/api/"
                            "environments/1-a47ac10b-58cc-4372-"
                            "a567-0e02b2c3d470"
                        }
                    },
                },
                "database": {
                    "id": 14,
                    "name": "db_name",
                    "_links": {
                        "self": {
                            "href": "https://cloud.acquia.com/api/"
                            "environments/1-a47ac10b-58cc-4372-a567-"
                            "0e02b2c3d470/databases/db_name"
                        }
                    },
                },
            },
        }
        mocker.register_uri("GET", uri, status_code=200, json=response)

        response = self.acquia.environment(env_id).backup_details(
            db_name, id_backup
        )
        self.assertEqual(response["id"], 1)
        self.assertIn("_embedded", response)

    def test_backup_download(self, mocker):
        env_id = "1-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        db_name = "db_name"
        id_backup = "1"
        uri = (
            f"{self.endpoint}/environments/{env_id}/"
            f"databases/{db_name}/backups/{id_backup}/actions/download"
        )

        response = {
            "url": "http://test-site.com/AH_DOWNLOAD?t=1&d=/mnt/files/site/"
            "backups/on-demand/backup.sql.gz&dev=hash",
            "expires_at": "2020-03-27T10:26:51+00:00",
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "1-a47ac10b-58cc-4372-a567-0e02b2c3d470/databases/"
                    "db_name/backups/1/actions/download"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "1-a47ac10b-58cc-4372-a567-0e02b2c3d470/databases/"
                    "db_name/backups/1/actions"
                },
            },
        }
        mocker.register_uri("GET", uri, status_code=200, json=response)

        response = self.acquia.environment(env_id).backup_download(
            db_name, id_backup
        )
        self.assertIn("url", response)

    def test_code_switch(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/environments/{env_id}/code/actions/switch"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)

        mocker.register_uri(
            "POST",
            uri,
            status_code=202,
            json={"message": "The code is being switched."},
        )

        response = self.acquia.environment(env_id).code_switch(
            "my-feature-branch"
        )

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"switched", response.content)

    def test_configure(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/environments/{env_id}"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)

        data = {
            "version": "5.5",
            "max_execution_time": 10,
            "memory_limit": 192,
            "apc": 128,
            "max_input_vars": 1000,
            "max_post_size": 256,
            "sendmail_path": "/usr/bin/sendmail",
            "varnish_over_ssl": False,
        }

        response_message = {
            "message": "The environment configuration is being updated."
        }

        mocker.register_uri("PUT", uri, status_code=202, json=response_message)

        response = self.acquia.environment(env_id).configure(data)

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"updated", response.content)

    def test_create_backup(self, mocker):
        env_id = "1-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        db_name = "db_name"
        uri = (
            f"{self.endpoint}/environments/{env_id}/"
            f"databases/{db_name}/backups"
        )

        response = {
            "message": "Creating the backup.",
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "12-d314739e-296f-11e9-b210-d663bd873d93/"
                    "databases/my_db/backups/"
                },
                "notification": {
                    "href": "https://cloud.acquia.com/api/notifications/"
                    "42b56cff-0b55-4bdf-a949-1fd0fca61c6c"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "12-d314739e-296f-11e9-b210-d663bd873d93/"
                    "databases/my_db/"
                },
            },
        }

        mocker.register_uri("POST", uri, status_code=202, json=response)
        response = self.acquia.environment(env_id).create_backup(db_name)
        self.assertEqual(response["message"], "Creating the backup.")
        self.assertIn("_links", response)

    def test_create_domain(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/environments/{env_id}/domains"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)

        response_message = {
            "message": "The domain 'ceruleanhq.com' is being added."
        }

        mocker.register_uri(
            "POST", uri, status_code=202, json=response_message
        )

        response = self.acquia.environment(env_id).create_domain(
            "ceruleanhq.com"
        )

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"added", response.content)

    def test_create_log_forwarding_destinations(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/environments/{env_id}/log-forwarding-destinations"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)

        response_message = {
            "message": "Log forwarding destination for the \
            environment has been created."
        }

        mocker.register_uri(
            "POST", uri, status_code=202, json=response_message
        )

        label = "Test destination"
        sources = ["apache-access", "apache-error"]
        consumer = "syslog"
        credentials = {
            "certificate": "-----BEGIN CERTIFICATE-----...\
            -----END CERTIFICATE-----"
        }
        address = "example.com:1234"

        response = self.acquia.environment(
            env_id
        ).create_log_forwarding_destinations(
            label, sources, consumer, credentials, address
        )

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"created", response.content)

    def test_delete_backup(self, mocker):
        env_id = "12-d314739e-296f-11e9-b210-d663bd873d93"
        db_name = "my_db"
        uri = (
            f"{self.endpoint}/environments/{env_id}/"
            f"databases/{db_name}/backups/1"
        )

        response = {
            "message": "Deleting the database backup.",
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/12-d314"
                    "739e-296f-11e9-b210-d663bd873d93/databases/"
                    "my_db/backups/1"
                },
                "notification": {
                    "href": "https://cloud.acquia.com/api/notifications/42b5"
                    "6cff-0b55-4bdf-a949-1fd0fca61c6c"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/12-d31"
                    "4739e-296f-11e9-b210-d663bd873d93/databases/"
                    "my_db/backups"
                },
            },
        }

        mocker.register_uri("DELETE", uri, status_code=202, json=response)
        response = self.acquia.environment(env_id).delete_backup(db_name, 1)
        self.assertEqual(response["message"], "Deleting the database backup.")
        self.assertIn("_links", response)

    def test_delete_domain(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        domain = "ceruleanhq.com"
        uri = "{base_uri}/environments/{env_id}/domains/{domain}"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id, domain=domain)

        mocker.register_uri("DELETE", uri, status_code=202)

        response = self.acquia.environment(env_id).delete_domain(
            "ceruleanhq.com"
        )

        self.assertEqual(response.status_code, 202)

    def test_clear_varnish_domain(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        domain = "ceruleanhq.com"
        uri = (
            "{base_uri}/environments/{env_id}/domains/"
            "{domain}/actions/clear-varnish"
        )
        uri = uri.format(base_uri=self.endpoint, env_id=env_id, domain=domain)

        response_message = {
            "message": "Varnish is being cleared for domain 'ceruleanhq.com'."
        }

        mocker.register_uri(
            "POST", uri, status_code=202, json=response_message
        )

        response = self.acquia.environment(env_id).clear_varnish_domain(
            "ceruleanhq.com"
        )

        self.assertEqual(response.status_code, 202)

    def test_clear_varnish_domains(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        domains = ["ceruleanhq.com"]
        uri = (
            "{base_uri}/environments/{env_id}/domains/" "actions/clear-varnish"
        )
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)

        response_message = {
            "message": "Varnish is being cleared for the selected domains."
        }

        mocker.register_uri(
            "POST", uri, status_code=202, json=response_message
        )

        response = self.acquia.environment(env_id).clear_varnish_domains(
            domains
        )

        self.assertEqual(response.status_code, 202)

    def test_delete_log_forwarding_destinations(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        destination_uuid = "df4c5428-8d2e-453d-9edf-e412647449b1"
        uri = (
            f"{self.endpoint}/environments/{env_id}/"
            f"log-forwarding-destinations/{destination_uuid}"
        )

        response_message = {
            "message": "Log forwarding destination has been deleted."
        }

        mocker.register_uri(
            "DELETE", uri, status_code=202, json=response_message
        )

        response = self.acquia.environment(
            env_id
        ).delete_log_forwarding_destinations(destination_uuid)

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"deleted", response.content)

    def test_destroy(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/environments/{env_id}"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)
        response = {"message": "The environment is being deleted."}

        mocker.register_uri(
            url=uri, method="DELETE", status_code=202, json=response
        )

        response = self.acquia.environment(env_id).destroy()

        self.assertEqual(response.status_code, 202)

    def test_deploy_code(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        env_id_from = "14-0c7e79ab-1c4a-424e-8446-76ae8be7e851"
        uri = "{base_uri}/environments/{env_id}/code"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)

        response_message = {"message": "The code is being deployed."}

        mocker.register_uri(
            "POST", uri, status_code=202, json=response_message
        )

        response = self.acquia.environment(env_id).deploy_code(env_id_from)

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"deployed", response.content)

    def test_deploy_database(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        env_id_from = "14-0c7e79ab-1c4a-424e-8446-76ae8be7e851"
        uri = "{base_uri}/environments/{env_id}/databases"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)

        response_message = {"message": "The database is queued for copying."}

        mocker.register_uri(
            "POST", uri, status_code=202, json=response_message
        )

        response = self.acquia.environment(env_id).deploy_database(
            env_id_from, "my_new_db"
        )

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"queued", response.content)

    def test_deploy_files(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        env_id_from = "14-0c7e79ab-1c4a-424e-8446-76ae8be7e851"
        uri = "{base_uri}/environments/{env_id}/files"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)

        response_message = {
            "message": "The files have been queued for copying."
        }

        mocker.register_uri(
            "POST", uri, status_code=202, json=response_message
        )

        response = self.acquia.environment(env_id).deploy_files(env_id_from)

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"queued", response.content)

    def test_get_crons(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/environments/{env_id}/crons"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)
        response_message = {
            "_embedded": {
                "items": [
                    {
                        "_links": {
                            "self": {
                                "href": "{baseUri}/environments/24-a47ac10b-58cc-4372-a567-0e02b2c3d470\
                                /crons/43595"
                            }
                        },
                        "command": "/usr/local/bin/drush -r /var/www/html/mysub/docroot \
                                   ah-db-backup mysub",
                        "day_month": "*",
                        "day_week": "*",
                        "environment": {
                            "id": "24-a47ac10b-58cc-4372-a567-0e02b2c3d470",
                            "name": "prod",
                        },
                        "flags": {
                            "enabled": True,
                            "on_any_web": True,
                            "system": True,
                        },
                        "hour": "8",
                        "id": "43595",
                        "label": None,
                        "minute": "0",
                        "month": "*",
                        "server": [],
                    },
                    {
                        "_links": {
                            "self": {
                                "href": "{baseUri}/environments/24-a47ac10b-58cc-4372-a567-0e02b2c3d470\
                                /crons/56834"
                            }
                        },
                        "command": "/usr/local/bin/drush9 --uri=[http://[site-uri] \
                                --root=/var/www/html/${AH_SITE_NAME}/docroot \
                                -dv cron &>> /var/log/sites/${AH_SITE_NAME}\
                                /logs/$(hostname -s)/drush-cron.log",
                        "day_month": "*",
                        "day_week": "*",
                        "environment": {
                            "id": "24-a47ac10b-58cc-4372-a567-0e02b2c3d470",
                            "name": "prod",
                        },
                        "flags": {
                            "enabled": True,
                            "on_any_web": True,
                            "system": False,
                        },
                        "hour": "*",
                        "id": "56834",
                        "label": "Site Cron Every Hour",
                        "minute": "0",
                        "month": "*",
                        "server": [],
                    },
                ]
            },
            "_links": {
                "parent": {
                    "href": "{baseUri}/environments/\
                            24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
                },
                "self": {
                    "href": "{baseUri}/environments/\
                            24-a47ac10b-58cc-4372-a567-0e02b2c3d470/crons"
                },
            },
            "total": 2,
        }

        mocker.register_uri("GET", uri, status_code=200, json=response_message)

        response = self.acquia.environment(env_id).get_crons()

        self.assertEqual(response["total"], 2)
        self.assertIn("_embedded", response)

    def test_get_log_forwarding_destinations(self, mocker):
        env_id = "5-185f07c7-9c4f-407b-8968-67892ebcb38a"
        uri = "{base_uri}/environments/{env_id}/log-forwarding-destinations"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)
        response_message = {
            "total": 2,
            "_links": {
                "self": {
                    "href": "{base_uri}/environments/5-185f07c7-9c4f-407b-8968-67892ebcb38a\
                    /log-forwarding-destinations"
                },
                "parent": {
                    "href": "{base_uri}/environments\
                    /5-185f07c7-9c4f-407b-8968-67892ebcb38a"
                },
            },
            "_embedded": {
                "items": [
                    {
                        "uuid": "df4c5428-8d2e-453d-9edf-e412647449b1",
                        "label": "Test destination",
                        "consumer": "sumologic",
                        "address": "example.com:1234",
                        "credentials": {
                            "certificate": {
                                "certificate": "-----BEGIN CERTIFICATE-----...\
                                -----END CERTIFICATE-----",
                                "expires_at": "2018-07-16T16:15:33+00:00",
                            },
                            "key": None,
                            "token": "4ded264c8891c400df8fc8905f07beb5f",
                        },
                        "sources": ["apache-access", "apache-error"],
                        "status": "active",
                        "flags": {
                            "enabled": True,
                            "certificate_expiring": False,
                        },
                        "environment": {
                            "id": "123-ea9060c5-1ed8-46ec-87d5-2ce2a0861577",
                            "name": "Test",
                        },
                    },
                    {
                        "uuid": "df4c5428-8d2e-453d-9edf-e412647449b5",
                        "label": "Another test destination",
                        "consumer": "syslog",
                        "address": "193.169.2.19:5678",
                        "credentials": {
                            "certificate": {
                                "certificate": "-----BEGIN CERTIFICATE-----...\
                                -----END CERTIFICATE-----",
                                "expires_at": "2018-07-16T16:15:33+00:00",
                            },
                            "key": "1d0789d519c0b943cf38f401d30ffbdcd2",
                            "token": "4ded264c8891c400df8fc8905f07beb5f",
                        },
                        "sources": ["drupal-request", "drupal-watchdog"],
                        "status": "active",
                        "flags": {
                            "enabled": False,
                            "certificate_expiring": True,
                        },
                        "environment": {
                            "id": "123-ea9060c5-1ed8-46ec-87d5-2ce2a0861577",
                            "name": "Test",
                        },
                    },
                ]
            },
        }

        mocker.register_uri("GET", uri, status_code=200, json=response_message)

        response = self.acquia.environment(
            env_id
        ).get_log_forwarding_destinations()

        self.assertEqual(response["total"], 2)
        self.assertIn("_embedded", response)

    def test_get_php_version(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/environments/{env_id}/"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)
        response_message = {"configuration": {"php": {"version": "7.2"}}}

        mocker.register_uri("GET", uri, status_code=200, json=response_message)

        response = self.acquia.environment(env_id).get_php_version()

        self.assertEqual(response["php_version"], "7.2")

    def test_get_servers(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/environments/{env_id}/servers"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)

        response_message = {
            "total": 2,
            "_links": {
                "self": {
                    "href": "{baseUri}/environments/"
                    "24-a47ac10b-58cc-4372-a567-0e02b2c3d470/servers"
                },
                "parent": {
                    "href": "{baseUri}/environments/"
                    "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
                },
            },
            "_embedded": {
                "items": [
                    {
                        "id": "6",
                        "name": "ded-6",
                        "hostname": "ded-6.servers.acquia.com",
                        "ssh_user": "user.dev",
                        "ip": "10.0.0.1",
                        "status": "normal",
                        "region": "us-west-1",
                        "roles": ["web", "db"],
                        "ami_type": "c1.medium",
                        "configuration": {
                            "memcache": 64,
                            "ecu": 5,
                            "memory": 1.7,
                        },
                        "flags": {
                            "elastic_ip": False,
                            "active_web": True,
                            "active_bal": False,
                            "primary_db": True,
                            "web": True,
                            "database": True,
                            "balancer": False,
                            "memcache": True,
                            "dedicated": False,
                            "self_service": False,
                        },
                        "environment": {
                            "id": "24-a47ac10b-58cc-4372-a567-0e02b2c3d470",
                            "name": "dev",
                        },
                        "_links": {
                            "self": {
                                "href": "{baseUri}/environments/"
                                "24-a47ac10b-58cc-4372-a567-"
                                "0e02b2c3d470/servers/6"
                            }
                        },
                    },
                    {
                        "id": "4",
                        "name": "bal-4",
                        "hostname": "bal-4.servers.acquia.com",
                        "ssh_user": None,
                        "ip": "10.0.0.2",
                        "status": "normal",
                        "region": "us-west-1",
                        "roles": ["bal"],
                        "ami_type": "m1.small",
                        "configuration": {
                            "memcache": None,
                            "ecu": 1,
                            "memory": 1.7,
                        },
                        "flags": {
                            "elastic_ip": False,
                            "active_web": False,
                            "active_bal": False,
                            "primary_db": True,
                            "web": False,
                            "database": False,
                            "balancer": True,
                            "memcache": False,
                            "dedicated": True,
                            "self_service": False,
                        },
                        "environment": {
                            "id": "24-a47ac10b-58cc-4372-a567-0e02b2c3d470",
                            "name": "dev",
                        },
                        "_links": {
                            "self": {
                                "href": "{baseUri}/environments/"
                                "24-a47ac10b-58cc-4372-a567-"
                                "0e02b2c3d470/servers/4"
                            }
                        },
                    },
                ]
            },
        }

        mocker.register_uri("GET", uri, status_code=200, json=response_message)

        response = self.acquia.environment(env_id).get_servers()

        self.assertEqual(response["total"], 2)
        self.assertIn("_embedded", response)

    def test_set_php_version(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        uri = "{base_uri}/environments/{env_id}"
        uri = uri.format(base_uri=self.endpoint, env_id=env_id)

        response_message = {
            "message": "The environment configuration is being updated."
        }

        mocker.register_uri("PUT", uri, status_code=202, json=response_message)

        response = self.acquia.environment(env_id).set_php_version("7.0")

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"updated", response.content)

    def test_update_log_forwarding_destinations(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        destination_uuid = "df4c5428-8d2e-453d-9edf-e412647449b1"
        uri = (
            f"{self.endpoint}/environments/{env_id}/"
            f"log-forwarding-destinations/{destination_uuid}"
        )

        response_message = {
            "message": "Log forwarding destination has been updated."
        }

        mocker.register_uri("PUT", uri, status_code=202, json=response_message)

        label = "Test destination"
        sources = ["apache-access", "apache-error"]
        consumer = "syslog"
        credentials = {
            "certificate": "-----BEGIN CERTIFICATE-----...\
            -----END CERTIFICATE-----"
        }
        address = "example.com:1234"

        response = self.acquia.environment(
            env_id
        ).update_log_forwarding_destinations(
            label, sources, consumer, credentials, address, destination_uuid
        )

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"updated", response.content)

    def test_enable_cron(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        cron_id = "1889"
        uri = (
            f"{self.endpoint}/environments/{env_id}/"
            f"crons/{cron_id}/actions/enable"
        )

        response = {
            "message": "The cron is being enabled.",
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "24-a47ac10b-58cc-4372-a567-0e02b2c3d470/crons/1889/"
                    "actions/enable"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "24-a47ac10b-58cc-4372-a567-0e02b2c3d470/"
                    "crons/1889/actions"
                },
                "notification": {
                    "href": "https://cloud.acquia.com/api/notifications/"
                    "ceda2e82-54b7-4181-ae97-6a3163b187b8"
                },
            },
        }
        mocker.register_uri("POST", uri, status_code=202, json=response)

        response = self.acquia.environment(env_id).enable_cron(cron_id)

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"The cron is being enabled.", response.content)

    def test_disable_cron(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        cron_id = "1234"
        uri = (
            f"{self.endpoint}/environments/{env_id}/"
            f"crons/{cron_id}/actions/disable"
        )

        response = {
            "message": "The cron is being disabled.",
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "24-a47ac10b-58cc-4372-a567-0e02b2c3d470/crons/1234/"
                    "actions/disable"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "24-a47ac10b-58cc-4372-a567-0e02b2c3d470/crons/"
                    "1234/actions"
                },
                "notification": {
                    "href": "https://cloud.acquia.com/api/notifications/"
                    "7b37b885-8ae4-454b-b8fa-ffaeff54f6a4"
                },
            },
        }
        mocker.register_uri("POST", uri, status_code=202, json=response)

        response = self.acquia.environment(env_id).disable_cron(cron_id)

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"The cron is being disabled.", response.content)

    def test_delete_cron(self, mocker):
        env_id = "24-a47ac10b-58cc-4372-a567-0e02b2c3d470"
        cron_id = "1891"
        uri = f"{self.endpoint}/environments/{env_id}/" f"crons/{cron_id}"

        response = {
            "message": "Deleting cron.",
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "24-a47ac10b-58cc-4372-a567-0e02b2c3d470/crons/1891"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "24-a47ac10b-58cc-4372-a567-0e02b2c3d470/crons"
                },
                "notification": {
                    "href": "https://cloud.acquia.com/api/notifications/"
                    "767cee8d-05f6-4761-a3dc-755957dfc9e6"
                },
            },
        }
        mocker.register_uri("DELETE", uri, status_code=202, json=response)

        response = self.acquia.environment(env_id).delete_cron(cron_id)

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"Deleting cron.", response.content)

    def test_get_ssl_settings(self, mocker):
        env_id = "3-110075c3-126e-6b43-c2ce-30be75fb33c2"
        uri = f"{self.endpoint}/environments/{env_id}/ssl"

        response = {
            "balancer": {"hostname": "example.us-east-1.elb.amazonaws.com"},
            "ips": ["127.0.0.1"],
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "3-110075c3-126e-6b43-c2ce-30be75fb33c2/ssl"
                },
                "certificates": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "3-110075c3-126e-6b43-c2ce-30be75fb33c2/ssl/certificates"
                },
                "csrs": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "3-110075c3-126e-6b43-c2ce-30be75fb33c2/ssl/csrs"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "3-110075c3-126e-6b43-c2ce-30be75fb33c2"
                },
            },
        }

        mocker.register_uri("GET", uri, status_code=200, json=response)
        response = self.acquia.environment(env_id).get_ssl_settings()
        self.assertIn("balancer", response)

    def test_get_ssl_certs(self, mocker):
        env_id = "5-a1a10dab-62f4-418c-bc58-ab7742078ba8"
        uri = f"{self.endpoint}/environments/{env_id}/ssl/certificates"

        response = {
            "total": 3,
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "5-a1a10dab-62f4-418c-bc58-ab7742078ba8/ssl/certificates"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "5-a1a10dab-62f4-418c-bc58-ab7742078ba8/ssl"
                },
            },
            "_embedded": {
                "items": [
                    {
                        "id": 7,
                        "label": None,
                        "certificate": "-----BEGIN CERTIFICATE-----...-----END CERTIFICATE-----",
                        "private_key": None,
                        "ca": "-----BEGIN CERTIFICATE-----...-----END CERTIFICATE-----",
                        "flags": {"active": True, "csr": True, "legacy": True},
                        "expires_at": "2022-03-28T00:12:34-0400",
                        "domains": ["example.com", "www.example.com"],
                        "environment": {
                            "id": "5-a1a10dab-62f4-418c-bc58-ab7742078ba8",
                            "name": "prod",
                        },
                        "_links": {
                            "self": {
                                "href": "https://cloud.acquia.com/api/environments/5-a1a10dab-62f4-418c-bc58-ab7742078ba8/ssl/certificates/7"
                            },
                            "csr": {
                                "href": "https://cloud.acquia.com/api/"
                                "environments/5-a1a10dab-62f4-418c-bc58-ab7742078ba8/ssl/csrs/7"
                            },
                        },
                    },
                    {
                        "id": 3,
                        "label": "Test Certificate 1",
                        "certificate": "-----BEGIN CERTIFICATE-----...-----END CERTIFICATE-----",
                        "private_key": "-----BEGIN RSA PRIVATE KEY-----...-----END RSA PRIVATE KEY-----",
                        "ca": "-----BEGIN CERTIFICATE-----...-----END CERTIFICATE-----",
                        "flags": {
                            "active": True,
                            "csr": False,
                            "legacy": False,
                        },
                        "expires_at": "2021-01-01T00:00:00-0400",
                        "domains": ["example2.com", "www.example2.com"],
                        "environment": {
                            "id": "5-a1a10dab-62f4-418c-bc58-ab7742078ba8",
                            "name": "prod",
                        },
                        "_links": {
                            "self": {
                                "href": "https://cloud.acquia.com/api/environments/5-a1a10dab-62f4-418c-bc58-ab7742078ba8/ssl/certificates/3"
                            }
                        },
                    },
                    {
                        "id": 4,
                        "label": "Test Certificate 2",
                        "certificate": "-----BEGIN CERTIFICATE-----...-----END CERTIFICATE-----",
                        "private_key": "-----BEGIN RSA PRIVATE KEY-----...-----END RSA PRIVATE KEY-----",
                        "ca": "-----BEGIN CERTIFICATE-----...-----END CERTIFICATE-----",
                        "flags": {
                            "active": False,
                            "csr": True,
                            "legacy": False,
                        },
                        "expires_at": "2021-01-01T00:00:00-0400",
                        "domains": ["example3.com", "www.example3.com"],
                        "environment": {
                            "id": "5-a1a10dab-62f4-418c-bc58-ab7742078ba8",
                            "name": "prod",
                        },
                        "_links": {
                            "self": {
                                "href": "https://cloud.acquia.com/api/environments/5-a1a10dab-62f4-418c-bc58-ab7742078ba8/ssl/certificates/4"
                            }
                        },
                    },
                ]
            },
        }

        mocker.register_uri("GET", uri, status_code=200, json=response)
        response = self.acquia.environment(env_id).get_ssl_certs()
        self.assertIn("certificate", response[0])

    def test_install_ssl_cert(self, mocker):
        env_id = "123-4ba86d4a-e193-4282-8963-d9d24746f444"
        uri = f"{self.endpoint}/environments/{env_id}/ssl/certificates"

        legacy = False
        certificate = (
            "-----BEGIN CERTIFICATE-----abc123....-----END CERTIFICATE-----",
        )
        private_key = (
            "-----BEGIN RSA PRIVATE KEY-----secret....-----END RSA PRIVATE KEY-----",
        )
        ca_certificates = (
            "-----BEGIN CERTIFICATE-----123abc....-----END CERTIFICATE-----",
        )
        csr_id = (123,)
        label = "My New Cert"

        response = {
            "message": "Installing the certificate.",
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/123-4ba86d4a-e193-4282-8963-d9d24746f444/ssl/certificates"
                },
                "notification": {
                    "href": "https://cloud.acquia.com/api/notifications/8fdacf25-38e4-4621-b5de-e78638fe2ceb"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/123-4ba86d4a-e193-4282-8963-d9d24746f444/ssl"
                },
            },
        }

        mocker.register_uri("POST", uri, status_code=202, json=response)

        response = self.acquia.environment(env_id).install_ssl_cert(
            label, certificate, private_key, ca_certificates, legacy, csr_id
        )

        self.assertEqual(response.status_code, 202)
        self.assertIn(b"Installing the certificate.", response.content)

    def test_get_ssl_cert(self, mocker):
        env_id = "5-9d46fd9d-e58b-47a3-8e9e-e8e0c2a854b4"
        cert_id = "13"
        uri = (
            f"{self.endpoint}/environments/{env_id}/ssl/certificates/{cert_id}"
        )

        response = {
            "id": 13,
            "label": "Test Certificate",
            "certificate": "-----BEGIN CERTIFICATE-----...-----END CERTIFICATE-----",
            "private_key": "-----BEGIN RSA PRIVATE KEY-----...-----END RSA PRIVATE KEY-----",
            "ca": "-----BEGIN CERTIFICATE-----...-----END CERTIFICATE-----",
            "flags": {"active": True, "csr": True, "legacy": False},
            "expires_at": "2022-03-28T00:12:34-0400",
            "domains": ["example.com", "www.example.com"],
            "environment": {
                "id": "5-9d46fd9d-e58b-47a3-8e9e-e8e0c2a854b4",
                "name": "prod",
            },
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "5-9d46fd9d-e58b-47a3-8e9e-e8e0c2a854b4/ssl/certificates/13"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "5-9d46fd9d-e58b-47a3-8e9e-e8e0c2a854b4/ssl/certificates"
                },
            },
        }

        mocker.register_uri("GET", uri, status_code=200, json=response)
        response = self.acquia.environment(env_id).get_ssl_cert(cert_id)
        self.assertIn("certificate", response)

    def test_delete_ssl_cert(self, mocker):
        env_id = "286-a027502b-ad6c-a48e-a7e8-aa0def7d25e1"
        cert_id = "9"
        uri = (
            f"{self.endpoint}/environments/{env_id}/ssl/certificates/{cert_id}"
        )

        response = {
            "message": "Deleting the certificate.",
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/286-a027502b-ad6c-a48e-a7e8-aa0def7d25e1/ssl/certificates/9"
                },
                "parent": {
                    "href": "https://cloud.acquia.com/api/environments/286-a027502b-ad6c-a48e-a7e8-aa0def7d25e1/ssl/certificates"
                },
                "notification": {
                    "href": "https://cloud.acquia.com/api/notifications/767cee8d-05f6-4761-a3dc-755957dfc9e6"
                },
            },
        }

        mocker.register_uri("DELETE", uri, status_code=202, json=response)
        response = self.acquia.environment(env_id).delete_ssl_cert(cert_id)
        self.assertEqual(response.status_code, 202)
        self.assertIn(b"Deleting the certificate.", response.content)

    def test_activate_ssl_cert(self, mocker):
        env_id = "123-a027502b-ad6c-a48e-a7e8-aa0def7d25e1"
        cert_id = "1"
        uri = f"{self.endpoint}/environments/{env_id}/ssl/certificates/{cert_id}/actions/activate"

        response = {
            "message": "Activating the certificate.",
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "123-a027502b-ad6c-a48e-a7e8-aa0def7d25e1/ssl/certificates/"
                    "1/actions/activate"
                },
                "notification": {
                    "href": "https://cloud.acquia.com/api/notifications/"
                    "4ee513c7-13b4-459f-af60-ba50c4f7cb5d"
                },
            },
        }

        mocker.register_uri("POST", uri, status_code=202, json=response)
        response = self.acquia.environment(env_id).activate_ssl_cert(cert_id)
        self.assertEqual(response.status_code, 202)
        self.assertIn(b"Activating the certificate.", response.content)

    def test_deactivate_ssl_cert(self, mocker):
        env_id = "123-a027502b-ad6c-a48e-a7e8-aa0def7d25e1"
        cert_id = "4547"
        uri = f"{self.endpoint}/environments/{env_id}/ssl/certificates/{cert_id}/actions/deactivate"

        response = {
            "message": "Deactivating the certificate.",
            "_links": {
                "self": {
                    "href": "https://cloud.acquia.com/api/environments/"
                    "123-a027502b-ad6c-a48e-a7e8-aa0def7d25e1/ssl/certificates/"
                    "4547/actions/deactivate"
                },
                "notification": {
                    "href": "https://cloud.acquia.com/api/notifications/"
                    "cb5de18e-5721-4c26-9f67-1a7d806dd09e"
                },
            },
        }

        mocker.register_uri("POST", uri, status_code=202, json=response)
        response = self.acquia.environment(env_id).deactivate_ssl_cert(cert_id)
        self.assertEqual(response.status_code, 202)
        self.assertIn(b"Deactivating the certificate.", response.content)



"""Module contains http hmac request, supports HTTP persistent connection."""

import httphmac
import requests


class HttpRequest(httphmac.Request):
    """Class to represent HTTP keep-alive hmac Request."""

    _session = None

    def __init__(self):
        """Initialize HTTP Request object with requests.Session."""
        super().__init__()
        self.session = self._get_session()

    def _get_session(self):
        """Generate new session object.

        :return: requests.Session
        """
        if not HttpRequest._session:
            HttpRequest._session = requests.Session()
        return HttpRequest._session

    def do(self):
        """ "Executes the request represented by this object.

        The requests library will be used for this purpose.
        Use requests.Session object for reuse TCP connection.
        Returns an instance of requests.Response.
        """
        data = None
        if self.body is not None and self.body != b"":
            data = self.body
        return self.session.request(
            self.method, str(self.url), data=data, headers=self.header
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Acquia Subscriptions resource"""

from acapi2.resources.acquialist import AcquiaList
from acapi2.resources.subscription import Subscription


class SubscriptionList(AcquiaList):
    def __init__(
        self, uri: str, api_key: str, api_secret: str, *args, **kwargs
    ) -> None:
        # TODO Filters
        super().__init__(uri, api_key, api_secret, *args, **kwargs)
        self.fetch()

    def fetch(self):
        subs = self.request(uri=self.uri).json()
        try:
            sub_items = subs["_embedded"]["items"]
        except KeyError:
            # TODO Handle this
            pass
        else:
            for sub in sub_items:
                name = sub["id"]
                subs_uri = "{base_uri}/{uuid}".format(
                    base_uri=self.uri, uuid=name
                )
                self.__setitem__(
                    name, Subscription(subs_uri, self.api_key, self.api_secret)
                )

    @property
    def base_uri(self) -> str:
        return self._base_uri

    @base_uri.setter
    def base_uri(self, base_uri: str):
        uri = "{}/subscriptions".format(base_uri)
        self._base_uri = uri



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Dictionary of Acquia Cloud API resources."""
import abc
import typing

from acapi2.exceptions import AcquiaCloudNoDataException
from acapi2.resources.acquiadata import AcquiaData


class AcquiaList(AcquiaData, dict):
    def __init__(
        self, base_uri: str, api_key: str, api_secret: str, *args, **kwargs
    ) -> None:
        self._sorted_keys = []  # type: typing.List[typing.Any]

        # Initialise the dict
        dict.__init__(self, *args, **kwargs)

        # Initialise the list
        self._base_uri: str = ""
        self.base_uri: str = base_uri  # type: ignore
        AcquiaData.__init__(self, self.base_uri, api_key, api_secret)

    def __delitem__(self, key):
        super(AcquiaList, self).__delitem__(key)
        self.sorted_keys = []

    def __setitem__(self, key, value):
        super(AcquiaList, self).__setitem__(key, value)
        self.sorted_keys = []

    def first(self) -> typing.Any:
        if not len(self):
            raise AcquiaCloudNoDataException("No data available.")

        key = self.search_pos(0)
        return self[key]

    def generate_resource_uri(self, resource: str) -> str:
        return f"{self.base_uri}/{resource}"

    def last(self):
        if not len(self):
            raise AcquiaCloudNoDataException("No data available.")

        key = self.search_pos(-1)
        return self[key]

    def search_pos(self, pos: int) -> str:
        keys = self.sorted_keys
        return keys[pos]

    @property
    def sorted_keys(self) -> list:
        if not self._sorted_keys:
            keys = list(self.keys())
            self._sorted_keys = sorted(keys)

        return self._sorted_keys

    @sorted_keys.setter
    def sorted_keys(self, keys: list):
        self._sorted_keys = keys

    @property  # type: ignore
    @abc.abstractmethod
    def base_uri(self) -> str:
        return self._base_uri

    @base_uri.setter  # type: ignore
    @abc.abstractmethod
    def base_uri(self, base_uri: str):
        self._base_uri = base_uri



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os

import requests_mock

from acapi2 import Acquia
from acapi2.exceptions import AcquiaCloudException
from acapi2.tests import BaseTest


@requests_mock.Mocker()
class TestAcquia(BaseTest):
    def test_failed_connection(self, mocker):
        pass

    def test_wrong_credentials_connection(self, mocker):
        pass

    def test_connection_system_health(self, mocker):
        pass

    def test_missing_os_credentials(self, mocker: object):
        os.environ["ACQUIA_CLOUD_API_KEY"] = ""
        os.environ["ACQUIA_CLOUD_API_SECRET"] = ""

        with self.assertRaises(AcquiaCloudException) as context:
            Acquia()

        self.assertEqual(str(context.exception), "Credentials not provided")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Agreement list"""

from acapi2.exceptions import AcquiaCloudNoDataException
from acapi2.resources.acquialist import AcquiaList
from acapi2.resources.agreement import Agreement


class AgreementList(AcquiaList):
    def __init__(
        self, uri: str, api_key: str, api_secret: str, *args, **kwargs
    ) -> None:

        super().__init__(uri, api_key, api_secret, *args, **kwargs)
        self.fetch()

    def fetch(self) -> None:
        agreements = self.request(uri=self.uri).json()
        try:
            agr_items = agreements["_embedded"]["items"]
        except KeyError:
            raise AcquiaCloudNoDataException()
        else:
            for agreement in agr_items:
                agr_uri = agreement["_links"]["self"]["href"]
                self.__setitem__(
                    agreement["uuid"],
                    Agreement(agr_uri, self.api_key, self.api_secret),
                )

    @property
    def base_uri(self) -> str:
        return self._base_uri

    @base_uri.setter
    def base_uri(self, base_uri: str):
        self._base_uri = f"{base_uri}/agreements"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Tasks tests"""

import requests_mock

from acapi2.resources.tasklist import TaskList
from acapi2.tests import BaseTest


@requests_mock.Mocker()
class TestTasks(BaseTest):
    def test_get_application_tasks(self, mocker):
        response = {
            "total": 2,
            "_links": {
                "self": {
                    "href": "{baseUri}/applications/0c7e79ab-1c4a"
                    "-424e-8446-76ae8be7e851/tasks"
                },
                "parent": {
                    "href": "{baseUri}/applications/0c7e79ab-1c4a"
                    "-424e-8446-76ae8be7e851"
                },
            },
            "_embedded": {
                "items": [
                    {
                        "progress": 0,
                        "user": {
                            "uuid": "a18a1d80-8896-11e1" "-9eb5-12313928d5b8",
                            "first_name": "James",
                            "last_name": "Kirk",
                            "mail": "james.kirk@example.com",
                            "picture_url": "https://accounts.acquia.com/"
                            "path/to/imagepng",
                            "username": "james.kirk",
                        },
                        "uuid": "53098518-0da8-4e41" "-943b-1198e3614f38",
                        "name": "OperationStarted",
                        "title": "Operation title",
                        "description": "Operation description.",
                        "created_at": "2017-04-04T17:21:53-04:00",
                        "started_at": "2017-04-04T17:21:53-04:00",
                        "completed_at": "2017-04-04T17:21:59-04:00",
                        "status": "started",
                        "type": "task",
                        "metadata": {
                            "environment": {
                                "ids": [
                                    "63-0c7e79ab-1c4a"
                                    "-424e-8446-76ae8be7e851"
                                ]
                            }
                        },
                        "labels": ["database", "hosting"],
                        "reference_uuid": "53098518-0da8-4e41"
                        "-943b-1198e3614f38",
                        "_links": {
                            "self": {
                                "href": "{baseUri}/applications/"
                                "0c7e79ab-1c4a-424e-8446"
                                "-76ae8be7e851/tasks"
                            }
                        },
                    },
                    {
                        "progress": 100,
                        "user": {
                            "uuid": "a18a1d80-8896-11e1" "-9eb5-12313928d5b8",
                            "first_name": "James",
                            "last_name": "Kirk",
                            "mail": "james.kirk@example.com",
                            "picture_url": "https://accounts.acquia.com/"
                            "path/to/imagepng",
                            "username": "james.kirk",
                        },
                        "uuid": "63098518-0da8-4e41" "-943b-1198e3614f38",
                        "name": "OperationStarted",
                        "title": "Operation title",
                        "description": "Operation description.",
                        "created_at": "2017-04-04T17:21:53-04:00",
                        "started_at": "2017-04-04T17:21:53-04:00",
                        "completed_at": "2017-04-04T17:21:59-04:00",
                        "status": "completed",
                        "type": "task",
                        "metadata": {
                            "environment": {
                                "ids": [
                                    "63-0c7e79ab-1c4a"
                                    "-424e-8446-76ae8be7e851"
                                ]
                            }
                        },
                        "labels": ["database", "hosting"],
                        "reference_uuid": "53098518-0da8-4e41"
                        "-943b-1198e3614f38",
                        "_links": {
                            "self": {
                                "href": "{baseUri}/applications/"
                                "0c7e79ab-1c4a-424e"
                                "-8446-76ae8be7e851/tasks"
                            }
                        },
                    },
                ]
            },
        }

        app_uuid = "0c7e79ab-1c4a-424e-8446-76ae8be7e851"
        uri = "{base_uri}/applications/{uuid}/tasks".format(
            base_uri=self.endpoint, uuid=app_uuid
        )
        mocker.register_uri("GET", uri, json=response, status_code=200)
        tasks = self.acquia.application(app_uuid).tasks()
        self.assertIsInstance(tasks, TaskList)

    def test_wait_task(self, mocker):
        pass

    def test_failed_task(self, mocker):
        pass

    def test_task_not_found(self, mocker):
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Acquia application resource."""
from requests.exceptions import RequestException
from requests.sessions import Session

from acapi2.resources.acquiaresource import AcquiaResource
from acapi2.resources.environment import Environment
from acapi2.resources.environmentlist import EnvironmentList
from acapi2.resources.notificationlist import NotificationList
from acapi2.resources.tasklist import TaskList


class Application(AcquiaResource):
    def artifacts(self):
        raise NotImplementedError

    def create_database(self, name: str) -> Session:
        uri = f"{self.uri}/databases"
        data = {"name": name}

        response = Session()
        try:
            response = self.request(uri=uri, method="POST", data=data)
        except RequestException:
            print("There was an error in the request.")

        return response

    def create_environment(
        self, label: str, env_name: str, databases: list = None
    ) -> Session:
        if not databases:
            databases = ["database_" + env_name]

        data = {"label": label, "branch": env_name, "databases": databases}

        uri = f"{self.uri}/environments"

        response = Session()
        try:
            response = self.request(uri=uri, method="POST", data=data)
        except RequestException:
            print("There was an error in the request.")

        return response

    def environments(
        self,
        filters: dict = None,
        sort: str = None,
        limit: int = None,
        offset: int = None,
    ) -> EnvironmentList:

        qry_params = {
            "filter": filters,
            "sort": sort,
            "limit": limit,
            "offset": offset,
        }

        return EnvironmentList(
            self.uri, self.api_key, self.api_secret, qry_params=qry_params
        )

    def environment(self, environment_id: str) -> Environment:
        uri = f"{self.uri}/{environment_id}"
        return Environment(uri, self.api_key, self.api_secret)

    def load(self) -> None:
        self.populate_data()

    def notifications(
        self,
        filters: dict = None,
        sort: str = None,
        limit: int = None,
        offset: int = None,
    ) -> NotificationList:
        """Get the notifications for this tasks."""

        qry_params = {
            "filter": filters,
            "sort": sort,
            "limit": limit,
            "offset": offset,
        }

        return NotificationList(
            self.uri, self.api_key, self.api_secret, qry_params=qry_params
        )

    def permissions(self) -> Session:
        uri = f"{self.uri}/permissions"

        response = Session()
        try:
            response = self.request(uri=uri, method="GET")
        except RequestException:
            print("There was an error in the request.")

        return response

    def tasks(self, filters: dict = None) -> TaskList:
        """DEPRECATED(use notifications): Get tasks for this application."""
        return TaskList(
            self.uri, self.api_key, self.api_secret, filters=filters
        )



#!/usr/bin/env python3
import os

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__name__), "README.md")) as f:
    long_description = f.read()

setup(
    name="acapi2",
    version="2.1.2",
    url="https://github.com/pmatias/python-acquia-cloud-2",
    download_url="https://pypi.org/project/acapi2",
    license="MIT",
    author="Matt Katz",
    author_email="matt@ceruleanhq.nl",
    description="Acquia Cloud API v2 client library.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    zip_safe=False,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Internet",
    ],
    platforms="any",
    python_requires=">=3.6",
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    install_requires=[
        "backoff>=1.10.0,<2",
        "http-hmac-python>=2.4.1,<3",
        "requests>=2.25.0,<=3.0",
        "requests-cache>=0.5.2,<0.9.2",
        "setuptools>=38.5",
    ],
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from acapi2.resources.acquiaresource import AcquiaResource
from acapi2.resources.applicationlist import ApplicationList


class Subscription(AcquiaResource):
    def applications(self):
        applications = ApplicationList(self.uri, self.api_key, self.api_secret)
        return applications

    def entitlements(self):
        raise NotImplementedError
